@@ -11,7 +11,7 @@ from steinbock.version import version as steinbock_version
 ilastik_binary = "/opt/ilastik/run_ilastik.sh"
 cellprofiler_binary = "cellprofiler"
 cellprofiler_plugin_dir = "/opt/cellprofiler_plugins"
-keras_models_dir = str(Path.home() / ".keras" / "models")
+keras_models_dir = "/opt/keras/models"
 
 
 def run_captured(args, *popen_args, file=sys.stdout, **popen_kwargs):
