@@ -20,8 +20,9 @@ def to_table_from_disk(*data_file_lists) -> pd.DataFrame:
                 left_index=True,
                 right_index=True,
             )
+        img_file = io.as_path_with_suffix(data_files[0], ".tiff")
         data_objs.append(data)
-        img_file_names.append(Path(data_files[0]).with_suffix(".tiff").name)
+        img_file_names.append(img_file.name)
     return pd.concat(data_objs, keys=img_file_names, names=["Image", "Object"])
 
 
