@@ -4,17 +4,20 @@ from pathlib import Path
 
 from steinbock import io
 from steinbock._env import check_steinbock_version
-from steinbock.measurement import data
+from steinbock.measurement.intensities import (
+    IntensityAggregation,
+    measure_intensities_from_disk,
+)
 
 
-_measurements = {
-    "sum": data.Measurement.SUM,
-    "min": data.Measurement.MIN,
-    "max": data.Measurement.MAX,
-    "mean": data.Measurement.MEAN,
-    "median": data.Measurement.MEDIAN,
-    "std": data.Measurement.STD,
-    "var": data.Measurement.VAR,
+_intensity_aggregations = {
+    "sum": IntensityAggregation.SUM,
+    "min": IntensityAggregation.MIN,
+    "max": IntensityAggregation.MAX,
+    "mean": IntensityAggregation.MEAN,
+    "median": IntensityAggregation.MEDIAN,
+    "std": IntensityAggregation.STD,
+    "var": IntensityAggregation.VAR,
 }
 
 
@@ -45,8 +48,10 @@ _measurements = {
 )
 @click.option(
     "--aggr",
-    "measurement_name",
-    type=click.Choice(list(_measurements.keys()), case_sensitive=True),
+    "intensity_aggregation_name",
+    type=click.Choice(
+        list(_intensity_aggregations.keys()), case_sensitive=True
+    ),
     default="mean",
     show_default=True,
     help="Function for aggregating cell pixels",
@@ -61,15 +66,18 @@ _measurements = {
 )
 @check_steinbock_version
 def intensities_cmd(
-    img_dir, mask_dir, panel_file, measurement_name, intensities_dir
+    img_dir, mask_dir, panel_file, intensity_aggregation_name, intensities_dir
 ):
     panel = io.read_panel(panel_file)
     channel_names = panel["name"].tolist()
     img_files = io.list_image_files(img_dir)
     mask_files = io.list_mask_files(mask_dir, base_files=img_files)
     Path(intensities_dir).mkdir(exist_ok=True)
-    for img_file, mask_file, intensities in data.measure_intensities_from_disk(
-        img_files, mask_files, channel_names, _measurements[measurement_name],
+    for img_file, mask_file, intensities in measure_intensities_from_disk(
+        img_files,
+        mask_files,
+        channel_names,
+        _intensity_aggregations[intensity_aggregation_name],
     ):
         intensities_stem = Path(intensities_dir) / img_file.stem
         intensities_file = io.write_data(
